from .types import ElementType

class Types:
    div = ElementType("div")
    p = ElementType("p")
    h1 = ElementType("h1")
    h2 = ElementType("h2")
    h3 = ElementType("h3")
    h4 = ElementType("h4")
    h5 = ElementType("h5")
    h6 = ElementType("h6")
    a = ElementType("a")
    button = ElementType("button")


from pymium.core import Space
from pymium.core.element import Element
def getElementById(space: Space, id: str) -> list[Element]:
    return _find_in_list_by_id(space.elements, id)

def _find_in_list_by_id(choosen_list: list, id: str) -> list:
    results = list()

    for element in choosen_list:

        if element.childs:

            results = results + _find_in_list_by_id(element.childs, id)

        if element.id == id:
            results.append(element)
    return results




from typing import List, Self, Optional
from pymium.core.element import Element
from pymium.core.base.style import Style
from pymium.core.assets.templates import templates
class Space:
    def __init__(self, title: str, style: Optional[Style] = None) -> None:
        self._elements: List[Element] = list()
        self.style = style or Style()
        self.title = title
        self._custom_style = ""
    
    @property
    def elements(self):
        return self._elements

    def append(self, Element):
        self._elements.append(Element)
    
    def _wrap_with_string(self, text: str) -> str:
        return f'"{text}"'
    
    def add_custom_css(self, css: str):
        self._custom_style += css

    def __str__(self) -> str:
        result: str = ""
        text = templates.base_html
        result = text.replace("{$space}", f"<body {' style = ' + self._wrap_with_string(str(self.style)) if str(self.style) else ''}>{'<style>'+ self._custom_style + '</style>' if self._custom_style else ''}{''.join([str(element) for element in self._elements])}</body>").replace("{$using pymium}", "").replace("{$title}", self.title)
        
        return result



# import webview

from pymium.core.base import Types
from pymium.core.element import Element, handler
from pymium.core.base.style import Style
from pymium.core import Space



from PyQt5.QtWidgets import QApplication, QMainWindow
from PyQt5 import QtWebEngineWidgets
from PyQt5 import QtCore
from PyQt5.QtCore import QObject, pyqtSlot, QVariant
from PyQt5.QtWebChannel import QWebChannel
import sys



class PymiumWindow(QMainWindow):
    def __init__(self, space: Space, title: str, width: int = 800, height: int = 600, frameless: bool = False, on_top: bool = False):
        super().__init__()

        self.setWindowTitle(title)
        self.setFixedWidth(width)
        self.setFixedHeight(height)
        self.view = QtWebEngineWidgets.QWebEngineView()
        html = str(space)
        self.view.setHtml(html)

        self.setWindowFlags(
            self.windowFlags() 
            | QtCore.Qt.FramelessWindowHint if frameless else self.windowFlags() 
            | QtCore.Qt.WindowStaysOnTopHint if on_top else self.windowFlags() 
        )
        self.setAttribute(QtCore.Qt.WA_TranslucentBackground)
        
        self.channel = QWebChannel()
        self.handler = CallHandler(space)
        self.channel.registerObject('handler', self.handler)
        self.view.page().setWebChannel(self.channel)
        # self.setWindowOpacity(0.5)
        self.setCentralWidget(self.view)
    def reload_html(self, space:Space):
        self.view.setHtml(str(space))


class CallHandler(QObject):
    def __init__(self, space: Space):
        super().__init__()
        self.space=space
    
    @pyqtSlot(QVariant, result=QVariant)
    def test1(self, args):
        element_data = str(args).split(":")
        element_name = element_data[0]
        element_id = element_data[1]    if element_data[2]!="noId" else None
        element_class = element_data[2] if element_data[2]!="noClass" else None

        id_elements = handler.getElementById(self.space, element_id)

        targets = []

        for element in id_elements:
          if element.className == element_class and str(element.elementType) == element_name:
            targets.append(element)

        for element in targets: 
            if element.onclick.__code__.co_argcount > 0:
                element.onclick(element)
            else:
                element.onclick()
        return "ok"

class PyWindow:
    def __init__(self, space:Space, width: int = 800, height: int = 600, always_on_top: bool = False, frameless: bool= False):
        self.app = QApplication(sys.argv)


        self.window = PymiumWindow(space, space.title, width, height, frameless, always_on_top)

    def run(self):
        
        self.window.show()


        self.app.exec()

    def set_html(self,space):
        self.window.reload_html(space) 



base_html = """{$using pymium}
<!DOCTYPE html>
<html lang="en">
<head>
    <meta charset="UTF-8">
    <meta http-equiv="X-UA-Compatible" content="IE=edge">
    <meta name="viewport" content="width=device-width, initial-scale=1.0">
    <title>{$title}</title>
    <script src="qrc:///qtwebchannel/qwebchannel.js"></script>
    </script>
    <script language="JavaScript">
        function sendtopy(text) {  
        new QWebChannel(qt.webChannelTransport, function (channel) {
          window.handler = channel.objects.handler;
          
          handler.test1(text)
        })};
    </script>
</head>
{$space}
</html>"""


from setuptools import setup, find_packages
setup(
name='pymium',
version='0.4.0',
author='Z3N',
author_email='iliya.souri7890@example.com',
description='Windows application creator',
packages=find_packages(),
classifiers=[
'Programming Language :: Python :: 3',
'License :: OSI Approved :: MIT License',
'Operating System :: OS Independent',
],
python_requires='>=3.11',
install_requires=["pywebview"]
)



from typing import Self


class Style:
    def __init__(self, **styles) -> None:
        self.styles = styles

    def add_style(self, **styles):
        self.styles = self.styles | styles

    def clone(self) -> Self:
        return Style(**self.styles)
        
    def __str__(self) -> None | str:
        if not self.styles:
            return ""
        results = ""
        for key in self.styles.keys():
            results += key.replace("_","-") + ":" + self.styles[key] + ";"
        return results
    


from pymium.core.base.types import ElementType
from pymium.core.base.style import Style
from typing import Optional, Self

class Element:
    def __init__(self,
                 elementType: ElementType,
                 id: Optional[str] = None,
                 className: Optional[str] = None,
                 innerHTML: Optional[str] = None,
                 style: Optional[Style] = None,
                 onclick  = None
                 ):
        self.elementType = elementType
        self.id = id
        self.className = className
        self.innerHTML = innerHTML
        self.style = style or Style()
        self._parent: Optional[Self] = None
        self._childs: list[Self] = []
        self.onclick = onclick
        self._att = dict()

    @property
    def parent(self) -> Optional[Self]:
        return self._parent
    
    @parent.setter
    def parent(self, parent:Self):
        self._parent = parent

    @property
    def childs(self) -> list[Self]:
        return self._childs
    
    def append(self, *elements: Self) -> None:
        for element in elements:
            element.parent = self.__class__
            self._childs.append(element)
    
    def _wrap_with_string(self, text: str) -> str:
        return f'"{text}"'
    
    def _wrap_with_single_string(self, text: str) -> str:
        return f"'{text}'"

    def add_attributes(self, **attributes):
        self._att = attributes | self._att
    
    def customid(self) -> str:
        idName = self.id or "noId"
        className = self.className or "noClass"
        elementType = self.elementType
        return f"{elementType}:{idName}:{className}"

    def __str__(self):
        return f"<{self.elementType}{' class = ' + self._wrap_with_string(self.className) if self.className else ''}{' id = ' + self._wrap_with_string(self.id) if self.id else ''}{' onclick = ' + self._wrap_with_string('sendtopy('+self._wrap_with_single_string(self.customid())+')') if self.onclick else ''}{' style = ' + self._wrap_with_string(str(self.style)) if str(self.style) else ''}{''.join([' ' + att + '=' + self._wrap_with_single_string(self._att[att]) for att in self._att.keys()])}>{self.innerHTML if self.innerHTML else ''}{''.join([str(element) for element in self._childs])}</{self.elementType}>"
